#
# spec file for package katacontainers
#
# Copyright (c) 2020 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


#
%define  kata_project          github.com/kata-containers

%ifarch x86_64
%define QEMUCMD qemu-system-x86_64
%else
%ifarch ppc64le
%define QEMUCMD qemu-system-ppc
%else
%define QEMUCMD qemu-system-%{_arch}
%endif
%endif

%define configPath %{_datarootdir}/defaults/kata-containers/
%define configACRN configuration-acrn.toml
%define configFC   configuration-fc.toml
%define configQEMU configuration-qemu.toml
%ifarch x86_64
# Note: braces used for bash brace expansion
%define defaultConfigFiles \{%{configACRN},%{configFC},%{configQEMU}\}
%else
%define defaultConfigFiles %{configQEMU}
%endif

Name:           katacontainers
Version:        1.10.0
Release:        bp153.1.118
Summary:        Kata Containers OCI container runtime
License:        Apache-2.0
Group:          System/Management
URL:            https://github.com/kata-containers
Source0:        runtime-%{version}.tar.xz
Source1:        proxy-%{version}.tar.xz
Source2:        shim-%{version}.tar.xz
Source3:        ksm-throttler-%{version}.tar.xz
Source4:        kata-fc
Source5:        kata-qemu
Source6:        katacontainers.rpmlintrc
ExclusiveArch:  x86_64 aarch64 ppc64le s390x
BuildRequires:  fdupes
BuildRequires:  golang(API) >= 1.12
Requires:       katacontainers-image-initrd  = %{version}
# Requires: are also required for build, to correctly detect the kernel
# version to use
BuildRequires:  katacontainers-image-initrd  = %{version}
# Required for build to correctly detect the kernel version to use
%ifarch x86_64
Requires:       qemu-x86
Recommends:     firecracker
%endif
%ifarch aarch64
Requires:       qemu-arm
%endif
%ifarch ppc64le
Requires:       qemu-ppc
%endif

%description
Kata Containers is an open source container runtime, building lightweight
virtual machines that seamlessly plug into the containers ecosystem.

%prep
%setup -q -c -a1 -a2 -a3

%build
export GOPATH=$HOME/go
rm -rf $HOME/go/src

suffix="-%{version}*"
dirs=($(ls -d {proxy,runtime,shim,ksm-throttler}${suffix}))

for d in ${dirs[@]}; do
    pkg=${d%$suffix}
    dest="$HOME/go/src/%{kata_project}/$pkg"
    mkdir -pv "$dest"
    cp    -avr "$d"/* "$dest"
done

cd $HOME/go/src/%{kata_project}/runtime
make %{?_smp_mflags} \
    SKIP_GO_VERSION_CHECK=1 QEMUCMD=%{QEMUCMD}

cd $HOME/go/src/%{kata_project}/proxy
make %{?_smp_mflags}

cd $HOME/go/src/%{kata_project}/shim
make %{?_smp_mflags}

cd $HOME/go/src/%{kata_project}/ksm-throttler
make %{?_smp_mflags} HAVE_SYSTEMD=yes all-installable

%check
# No tests

%install
export GOPATH=$HOME/go

cd $HOME/go/src/%{kata_project}/runtime
make \
    SKIP_GO_VERSION_CHECK=1 \
    DESTDIR=%{buildroot} \
    PREFIX=%{_prefix} \
    LIBEXECDIR=%{_libexecdir} \
    QEMUCMD=%{QEMUCMD} \
    install

# Only initrd is supported: delete the "image =" entries only in files where
# both "image =" and "initrd =" is specified
for f in %{buildroot}%{configPath}/%{defaultConfigFiles}; do
  grep -q "^image =" "$f" && grep -q "^initrd = " "$f" && sed -i -E -e '/^image =/d' $f
done

# Replace /usr/libexec path with /usr/lib
sed -i -E \
    -e "s,/usr/libexec,%{_libexecdir}," \
    -e "s,^kernel =.*$,kernel = \"%{_datarootdir}/kata-containers/vmlinuz\"," \
    %{buildroot}%{configPath}/%{defaultConfigFiles}

install -m 644 -D %{buildroot}%{configPath}/configuration.toml %{buildroot}%{_sysconfdir}/kata-containers/configuration.toml

cd $HOME/go/src/%{kata_project}/proxy
make \
    DESTDIR=%{buildroot} \
    PREFIX=%{_prefix} \
    LIBEXECDIR=%{_libexecdir} \
    install

cd $HOME/go/src/%{kata_project}/shim
make \
    DESTDIR=%{buildroot} \
    PREFIX=%{_prefix} \
    LIBEXECDIR=%{_libexecdir} \
    install

cd $HOME/go/src/%{kata_project}/ksm-throttler
make \
    DESTDIR=%{buildroot} \
    PREFIX=%{_prefix} \
    LIBEXECDIR=%{_libexecdir} \
    HAVE_SYSTEMD=yes \
    UNIT_DIR=%{_unitdir} \
    V=1 \
    install

sed -i -E -e 's,/usr/libexec,%{_libexecdir},' \
    %{buildroot}/%{_unitdir}/kata-ksm-throttler.service \
    %{buildroot}/%{_unitdir}/kata-vc-throttler.service

# symlink /usr/sbin/rcFOO -> /usr/sbin/service
install -d %{buildroot}%{_sbindir}
ln -s service %{buildroot}%{_sbindir}/rckata-ksm-throttler
ln -s service %{buildroot}%{_sbindir}/rckata-vc-throttler

# install kata runtime helpers
install -m 755 %{SOURCE4} %{buildroot}%{_bindir}/kata-fc
install -m 755 %{SOURCE5} %{buildroot}%{_bindir}/kata-qemu

%fdupes %{buildroot}/%{_prefix}

%pre
%service_add_pre kata-ksm-throttler.service
%service_add_pre kata-vc-throttler.service

%post
%service_add_post kata-ksm-throttler.service
%service_add_post kata-vc-throttler.service

%preun
%service_del_preun kata-ksm-throttler.service
%service_del_preun kata-vc-throttler.service

%postun
%service_del_postun kata-ksm-throttler.service
%service_del_postun kata-vc-throttler.service

%files
%defattr(-,root,root,-)
# Runtime
# Binaries
%{_bindir}/kata-runtime
%{_bindir}/kata-fc
%{_bindir}/kata-qemu
%{_bindir}/containerd-shim-kata-v2
%dir %{_libexecdir}/kata-containers
%{_libexecdir}/kata-containers/kata-netmon
%{_bindir}/kata-collect-data.sh
# Manpages
# Configs
%dir %{_sysconfdir}/kata-containers
%config(noreplace) %{_sysconfdir}/kata-containers/configuration.toml
# Default configs
%dir %{_datarootdir}/defaults
%dir %{_datarootdir}/defaults/kata-containers
%config %{_datarootdir}/defaults/kata-containers/configuration.toml
%config %{_datarootdir}/defaults/kata-containers/configuration-*.toml
# Completion
%{_datarootdir}/bash-completion/completions/kata-runtime

# proxy
# Binaries
%{_libexecdir}/kata-containers/kata-proxy

# shim
# Binaries
%{_libexecdir}/kata-containers/kata-shim

# ksm-throttler
%dir %{_libexecdir}/kata-ksm-throttler
%dir %{_libexecdir}/kata-ksm-throttler/trigger
%dir %{_libexecdir}/kata-ksm-throttler/trigger/virtcontainers
# Binaries
%{_libexecdir}/kata-ksm-throttler/kata-ksm-throttler
%{_libexecdir}/kata-ksm-throttler/trigger/virtcontainers/vc
%{_sbindir}/rckata-ksm-throttler
%{_sbindir}/rckata-vc-throttler
# systemd
%{_unitdir}/kata-ksm-throttler.service
%{_unitdir}/kata-vc-throttler.service

%license runtime-%{version}/LICENSE

%changelog
* Mon Jan 20 2020 Ralf Haferkamp <rhafer@suse.com>
- Update to 1.10.0:
  runtime:
  - Initial support for Cloud Hypervisor
  - HybridVsock support for cloud hypervisor and firecracker
  - Updated Firecracker version to v0.19.1
  - Better rootless support for firecracker
  - This release deprecates bridged networking model
* Fri Dec 13 2019 Ralf Haferkamp <rhafer@suse.com>
- Update to 1.9.3:
  runtime:
  - versions: bump fc version to v0.18.1
  - backport 1.9: fix wrong number cpus after killing a container
  - virtcontainers/store: make VCStoreUUIDPath rootless
  - vc: Don't adjust block index on error
  - vc: Persist file handle may leak in FS#ToDisk
* Wed Dec  4 2019 Ralf Haferkamp <rhafer@suse.com>
- Update to 1.9.2:
  runtime:
  - rootless: Fix rangeUID parsing (Backport to 1.9)
  - rootless: Fix cgroup creation logic for rootless(Backport to 1.9)
* Thu Nov 14 2019 Ralf Haferkamp <rhafer@suse.com>
- Update to 1.9.1:
  runtime:
  - sandbox/cgroups: don't constrain if using SandboxCgroupsOnly
  - ci: Fix versions_checker.sh
* Thu Oct 24 2019 Ralf Haferkamp <rhafer@suse.com>
- Update to 1.9.0 tarballs, this is just a version bump without
  any code changes.
* Mon Oct 21 2019 Ralf Haferkamp <rhafer@suse.com>
- Update to 1.9.0~rc0:
  runtime:
  - Fix cache factory UT
  - Virtio-fs v0.3 support
  - virtcontainers: set agent's logs vsock port
  - config: Fix `virtio-fs` typo in Makefile
  - Hypervisor: UUID fix for acrn hypevisor
  - virtcontainers: change firecracker socket permissions
  - Add annotations to provide custom configs
  - Fix CRIO + Firecracker
  - rootless: add rootless to kata
  - QEMU: do not require nvdimm machine option with initrd
  - s390x: Fix runtime build for s390x
  - versions: Update kernel to 4.19.75
  - config: honor DEFSHAREDFS_QEMU_VIRTIOFS and CONFIG_QEMU_VIRTIOFS_IN
  - Support Firecracker 0.18
  - virtcontainers: fix the issue of missing qemu error logs
  - config: Fix the qemu-virtiofs.toml
  - s390x: Share image between qemu instances
  - The unit of newMemory is MB
  - config: use 9p as default shared filesystem for nemu
  - Remove annotation config json key
  shim:
  - shim/firecracker: Read agent's logs
  - vendor: update kata agent
- Fix config for set-version source-service to set version based
  on the runtime filename. The previously used "katacontainers"
  caused the service to fail.
* Wed Sep 25 2019 Marco Vedovati <mvedovati@suse.com>
- Update katacontainers to version 1.9.0~alpha2:
  runtime 1.9.0~alpha2:
  * virtcontainers: Set sandbox dns in sandbox request
  * vc: Remove bind destination when unmounting
  * virtio-fs: add virtio_fs_extra_args for virtiofsd
  * sandbox: combine sandbox cgroup functions
  * sandbox: Join cgroup sandbox on create.
  * cgroups: container: check cgroup path before use it
  * sandbox: cgroup: prefix cgroup related methods
  * sandbox: cgroups: move methods to sandbox file
  * hypervisor: Fix MacVTap internetworking support in ACRN
  * CompatOCISpec: limit usage of CompatOCISpec
  * s390x: add virtio-blk-ccw
  * virtcontainers: create generic function
  * vsock: set VHOST_VSOCK_SET_GUEST_CID based based on arch
  * vc: Delete store when new/create container is failed
  * virtcontainers: fix kernel modules annotations
  * kata-check: require kvm/vhost modules for amd64
  * kata-check: reduce default output verbosity
  * v2: Prevent killing all container processes when exec is failed
  * shim-v2: add network stat in metric
  * qemu: fix error message miss
  * monitor: enlarge watch buffer
  * hypervisor: allow to return a slice of pids
  * virtcontainers: Fix the issue of watching console for firecracker
  * qemu: add logfile when debug is on
  * qemu: fix memory prealloc option handling
  * qemu: check guest status with qmp query-status
  * shimv2: cancel monitor before stopping sandbox
  * qemu: do not try to stop qemu multiple times
  * network: always cold unplug network devices
  * Revert: "sandbox: remove network before stopping vm"
  * agent: add default timeout for grpc requests
  * container: do not pause a StateReady container
  * sandbox: remove network before stopping vm
  * virtcontainers: fix hotplug pci devices execeed max capacity bug
  * vsock: Propogate error for vsock ioctl
  * network: Ignore routes with proto as "kernel"
  * acrn: Change the default network model for ACRN to macvtap
  * network: Deprecate bridged networking mode.
  * network: fix failed to remove network
  * virtcontainers: add support for loading kernel modules
  * shim-v2: fix shim leak when hypervisor exit unexpectly
  * virtiofs: wait for virtiofsd process to release its resources
  * pkg/katautils: Do not set init in the kernel command line
  * virtiofs: fix virtiofs crash when cache=none
  * virtcontainers: support SMP die
  * qemu: support vfio pass x-pci-vendor-id and x-pci-device-id pass
  * ut: skip TestBindUnmountContainerRootfsENOENTNotError for non-root
  * ut: skip TestStartNetworkMonitor for non-root
  * vc: move container mount cleanup to container.go
  * shimv2: monitor sandbox liveness
  * monitor: watch hypervisor
  * agent: use hypervisor pid as backup proxy pid for non-kata proxy cases
  * sandbox: do not fail SIGKILL
  * sandbox: support force stop
  * agent: mark agent dead when failing to connect
  * container: allow to stop a paused container
* Wed Sep  4 2019 Marco Vedovati <mvedovati@suse.com>
- Add config file in /etc/kata-containers
- Build with golang >= 1.12
* Tue Aug 27 2019 Marco Vedovati <mvedovati@suse.com>
- Remove image entry from configuration files only where both image and
  initrd are supported.
* Thu Aug  8 2019 Marco Vedovati <mvedovati@suse.com>
- Move dependency on kernel-kvmsmall to the katacontainers-image package.
* Fri Aug  2 2019 Marco Vedovati <mvedovati@suse.com>
- Set the kernel version in the configuration files according to the one
  specified in the initrd image name.
* Wed Jul 31 2019 Marco Vedovati <mvedovati@suse.com>
- Use kernel-kvmsmall as guest VM kernel
- Use katacontainers-image-initrd as gues VM OS image
- Update katacontainers to version 1.9.0~alpha:
  - Introduce ACRN hypervisor support
  - Upgrade the QEMU hypervisor from a QEMU-lite base to upstream QEMU 4.0.
  - Kata templating code to make use of the upstream x-ignored-shared.
  - Firecracker hypervisor is updated to 0.17
  - Kata now has support for using Firecracker's jailer
  - Fixes and usability improvements for virtio-fs
* Wed May 29 2019 Marco Vedovati <mvedovati@suse.com>
- Do not require or  recomment qemu-lite, upstream is switching to
  QEMU 4.0
* Tue May 28 2019 Marco Vedovati <mvedovati@suse.com>
- Do not require qemu-lite, just recommend it
* Tue May 28 2019 Marco Vedovati <mvedovati@suse.com>
- Update katacontainers to version 1.7.0:
  + early experimental preview of virtio-fs
  - virtio-fs is being developed as a replacement for 9pfs, providing better
    performance and compatibility for workloads which require host to guest
    filesystem sharing
  * improved the implementation of k8s empty-dir (ephemeral volumes) based
  on host directories by creating these inside the VM itself.
  * support for firecracker version 0.16
  * a few optimization fixes, updated versions support
* Wed Apr 24 2019 Marco Vedovati <mvedovati@suse.com>
- Update katacontainers to version 1.7.0~alpha1:
  runtime:
  * agent: pass correct mount type to agent for ephemeral volumes
  * network: Make tcfilter model as default
  * netmon: Fix bug in how routes are converted
  * storage: create k8s emptyDir inside VM
  * cli: fix kata-check test
  * shimv2: fix the issue of stop container failed
  * qemu: Remove the storage directories if qemu get from the factory
  * runtime: support memory hotplug via probe interface on aarch64
  shim:
  proxy:
  ksm-throttler:
  - no changes
* Mon Mar 25 2019 Flavio Castelli <fcastelli@suse.com>
- Update katacontainers to version 1.6.0:
  - Add OpenTracing support: this release includes preview changes to support OpenTracing
  - As part of the changes to enabled virtio-fs, now the agent can mount virtio-fs shared directories
  - The agent allows cpuset request that not match with vcpus in the guest
  - Kata Containers project updated to the Linux Kernel 4.19.x as preferred kernel version
  - Now Kata supports NVDIMM on arm64
  - Now cpu cgroups in sandbox are honored: This includes user defined paths and limit hypervisor vCPU threads
- runtime:
  - Kata now uses kernel: update to 4.19.x as preferred kernel version.
  - Now Kata support snvdimm on arm64
  - Reimplement sandbox cgroup
  - Now cpu related cgroups in sandbox are honored including use user defined paths and limit hypervisor vcpu threads
* Fri Mar 22 2019 Flavio Castelli <fcastelli@suse.com>
- Add runtime helpers to start either the qemu or the firecracker
  backends
* Wed Jan 30 2019 Marco Vedovati <mvedovati@suse.com>
- Update to Kata Containers version 1.5.0
  runtime:
  + add firecracker support
  + containerd shim v2 support
  * kata-check: do not require nested virtualization
  + kata-env: Show runtime trace setting
  + factory: set guest time after resuming
  + sandbox: cleanup sandbox if creation failed
  + block: Add new block storage driver "nvdimm"
  + s390x: add support for s390x
  shim:
  + tracing: Add opentracing support
* Mon Dec 17 2018 Marco Vedovati <mvedovati@suse.com>
- katacontainers package creation version 1.5.0~rc1
